﻿local L = LibStub:GetLibrary("AceLocale-3.0"):GetLocale("Grid2")
local DBL = LibStub:GetLibrary("LibDBLayers-1.0")

local Grid2StatusWeaponBuff = Grid2:NewModule("StatusWeaponBuff")

local MainHand = Grid2.statusPrototype:new("buff-mainhand")
local OffHand = Grid2.statusPrototype:new("buff-offhand")

Grid2 = Grid2

local EnableUpdate
do
	local objectTimer
	local objects = {}
	local frame
	local count = 0

	local function f()
		for object, param in pairs(objects) do
			object:UpdatePeriodic(param)
		end
	end
	function EnableUpdate(object, enable, param)
		local prevInactive = (count == 0)

		if (enable) then
			objects[object] = param or true
			count = count + 1
		else
			objects[object] = nil
			count = count - 1
		end
		assert(count >= 0)
		local currInactive = (count == 0)
		if (prevInactive ~= currInactive) then
			if (currInactive) then
				Grid2:CancelTimer(objectTimer)
			else
				objectTimer = Grid2:ScheduleRepeatingTimer(f, 1)
			end
		end
	end
end

local oldMainHandEnchant, oldOffHandEnchant, oldMainHandCharges, oldOffHandCharges, oldMainHandTexture, oldOffHandTexture



function MainHand:UpdatePeriodic()
	local hasMainHandEnchant, mainHandExpiration, mainHandCharges, hasOffHandEnchant, offHandExpiration, offHandCharges = GetWeaponEnchantInfo()
	local texture = GetInventoryItemTexture("player", 16)
	if (hasMainHandEnchant ~= oldMainHandEnchant or mainHandCharges ~= oldMainHandCharges or texture ~= oldMainHandTexture) then
		oldMainHandEnchant = hasMainHandEnchant
		oldMainHandCharges = mainHandCharges
		oldMainHandTexture = texture
		self:UpdateIndicators("player")
	end
end

function MainHand:OnEnable()
	EnableUpdate(self, true)
end

function MainHand:OnDisable()
	EnableUpdate(self)
end

function MainHand:Active(unit)
	if (unit ~= "player") then
		return false
	end
	local hasMainHandEnchant, mainHandExpiration, mainHandCharges, hasOffHandEnchant, offHandExpiration, offHandCharges = GetWeaponEnchantInfo()
	return hasMainHandEnchant
end

function MainHand:Inactive(unit)
	if (unit ~= "player") then
		return false
	end
	local hasMainHandEnchant, mainHandExpiration, mainHandCharges, hasOffHandEnchant, offHandExpiration, offHandCharges = GetWeaponEnchantInfo()
	return not hasMainHandEnchant
end

function MainHand:GetColor(unit)
	local color = self.dbx.color1
	return color.r, color.g, color.b, color.a
end

function MainHand:GetIcon(unit)
	local texture = GetInventoryItemTexture("player", 16)
	return texture
end

function MainHand:GetCount(unit)
	local hasMainHandEnchant, mainHandExpiration, mainHandCharges, hasOffHandEnchant, offHandExpiration, offHandCharges = GetWeaponEnchantInfo()
	return mainHandCharges
end

function MainHand:GetText(unit)
	return L["MainHand"]
end

function MainHand:UpdateDB()
	local missing = self.dbx.missing
	self.IsActive = missing and MainHand.Inactive or MainHand.Active
	-- self.GetIcon = missing and status_GetIconMissing or status_GetIcon
end

local function Create(baseKey, dbx)
	Grid2:RegisterStatus(MainHand, {"color", "icon", "text"}, baseKey, dbx)
	MainHand:UpdateDB()

	return MainHand
end

Grid2.setupFunc["buff-mainhand"] = Create



function OffHand:UpdatePeriodic()
	local hasMainHandEnchant, mainHandExpiration, mainHandCharges, hasOffHandEnchant, offHandExpiration, offHandCharges = GetWeaponEnchantInfo()
	local texture = GetInventoryItemTexture("player", 17)
	if (hasOffHandEnchant ~= oldOffHandEnchant or offHandCharges ~= oldOffHandCharges or texture ~= oldOffHandTexture) then
		oldOffHandEnchant = hasOffHandEnchant
		oldOffHandCharges = offHandCharges
		oldOffHandTexture = texture
		self:UpdateIndicators("player")
	end
end

function OffHand:OnEnable()
	EnableUpdate(self, true)
end

function OffHand:OnDisable()
	EnableUpdate(self)
end

function OffHand:Active(unit)
	if (unit ~= "player") then
		return false
	end
	local hasMainHandEnchant, mainHandExpiration, mainHandCharges, hasOffHandEnchant, offHandExpiration, offHandCharges = GetWeaponEnchantInfo()
	return hasOffHandEnchant
end

function OffHand:Inactive(unit)
	if (unit ~= "player") then
		return false
	end
	local hasMainHandEnchant, mainHandExpiration, mainHandCharges, hasOffHandEnchant, offHandExpiration, offHandCharges = GetWeaponEnchantInfo()
	return not hasOffHandEnchant
end

function OffHand:GetColor(unit)
	local color = self.dbx.color1
	return color.r, color.g, color.b, color.a
end

function OffHand:GetCount(unit)
	local hasMainHandEnchant, mainHandExpiration, mainHandCharges, hasOffHandEnchant, offHandExpiration, offHandCharges = GetWeaponEnchantInfo()
	return offHandCharges
end

function OffHand:GetIcon(unit)
	local texture = GetInventoryItemTexture("player", 17)
	return texture
end

function OffHand:GetText(unit)
	return L["OffHand"]
end

function OffHand:UpdateDB()
	local missing = self.dbx.missing
	self.IsActive = missing and OffHand.Inactive or OffHand.Active
	-- self.GetIcon = missing and status_GetIconMissing or status_GetIcon
end

local function Create(baseKey, dbx)
	Grid2:RegisterStatus(OffHand, {"color", "icon", "text"}, baseKey, dbx)
	OffHand:UpdateDB()

	return OffHand
end

Grid2.setupFunc["buff-offhand"] = Create




local currentCodeVersion = {
	"class", 1,
}
-- Hook the loading of options so our associated lod options get loaded at the right time.
local prev_LoadOptions = Grid2.LoadOptions
function Grid2:LoadOptions(dblData, ...)
	local upgrade = prev_LoadOptions(self, dblData, ...)

	upgrade = DBL:LoadOptions("Grid2StatusWeaponBuffOptions", dblData, nil, unpack(currentCodeVersion)) or upgrade

	return upgrade
end

-- Hook UpgradeDefaults to blend in default options if current ones are old.
local prev_UpgradeDefaults = Grid2.UpgradeDefaults
function Grid2:UpgradeDefaults(dblData, ...)
	local flatten = prev_UpgradeDefaults(self, dblData, ...)

	local Grid2StatusWeaponBuffOptions = Grid2Options.plugins["Grid2StatusWeaponBuffOptions"]
	if (Grid2StatusWeaponBuffOptions) then
		flatten = DBL:UpgradeDefaults("Grid2StatusWeaponBuffOptions", dblData, Grid2StatusWeaponBuffOptions.UpgradeDefaults, unpack(currentCodeVersion)) or flatten
	end
	
	return flatten
end

